linux-arm-kernel.lists.infradead.org archive mirror
 help / color / mirror / Atom feed
From: Cristian Marussi <cristian.marussi@arm.com>
To: linux-kernel@vger.kernel.org, linux-arm-kernel@lists.infradead.org
Cc: f.fainelli@gmail.com, vincent.guittot@linaro.org,
	sudeep.holla@arm.com, thara.gopinath@linaro.org,
	cristian.marussi@arm.com, james.quinlan@broadcom.com,
	Jonathan.Cameron@Huawei.com, souvik.chakravarty@arm.com,
	etienne.carriere@linaro.org, lukasz.luba@arm.com
Subject: [PATCH v6 04/37] firmware: arm_scmi: add devm_acquire_protocol helper
Date: Tue,  2 Feb 2021 22:15:22 +0000	[thread overview]
Message-ID: <20210202221555.41167-5-cristian.marussi@arm.com> (raw)
In-Reply-To: <20210202221555.41167-1-cristian.marussi@arm.com>

Add a method to get hold of a protocol, causing it to be initialized and
its resource accounting updated, without getting access to its operations
and handle.

Some protocols, like SCMI SystemPower, do not expose any protocol ops to
the Kernel OSPM agent but still need to be at least initialized: this
helper avoids the need to invoke a full devm_get_protocol() only to get
the protocol initialized while throwing away unused the protocol ops and
handle.

Signed-off-by: Cristian Marussi <cristian.marussi@arm.com>
---
 drivers/firmware/arm_scmi/driver.c | 70 +++++++++++++++++++++++-------
 include/linux/scmi_protocol.h      | 14 ++++--
 2 files changed, 66 insertions(+), 18 deletions(-)

diff --git a/drivers/firmware/arm_scmi/driver.c b/drivers/firmware/arm_scmi/driver.c
index 583c00a1355b..ca911a7eb511 100644
--- a/drivers/firmware/arm_scmi/driver.c
+++ b/drivers/firmware/arm_scmi/driver.c
@@ -745,6 +745,30 @@ static void scmi_devm_release_protocol(struct device *dev, void *res)
 	scmi_release_protocol(dres->handle, dres->protocol_id);
 }
 
+static struct scmi_protocol_instance __must_check *
+__scmi_devres_get_protocol_instance(struct scmi_device *sdev, u8 protocol_id)
+{
+	struct scmi_protocol_devres *dres;
+	struct scmi_protocol_instance *pi;
+
+	dres = devres_alloc(scmi_devm_release_protocol,
+			    sizeof(*dres), GFP_KERNEL);
+	if (!dres)
+		return ERR_PTR(-ENOMEM);
+
+	pi = scmi_get_protocol_instance(sdev->handle, protocol_id);
+	if (IS_ERR(pi)) {
+		devres_free(dres);
+		return pi;
+	}
+
+	dres->handle = sdev->handle;
+	dres->protocol_id = protocol_id;
+	devres_add(&sdev->dev, dres);
+
+	return pi;
+}
+
 /**
  * scmi_devm_get_protocol  - Devres managed get protocol operations and handle
  * @sdev: A reference to an scmi_device whose embedded struct device is to
@@ -768,32 +792,47 @@ scmi_devm_get_protocol(struct scmi_device *sdev, u8 protocol_id,
 		       struct scmi_protocol_handle **ph)
 {
 	struct scmi_protocol_instance *pi;
-	struct scmi_protocol_devres *dres;
-	struct scmi_handle *handle = sdev->handle;
 
 	if (!ph)
 		return ERR_PTR(-EINVAL);
 
-	dres = devres_alloc(scmi_devm_release_protocol,
-			    sizeof(*dres), GFP_KERNEL);
-	if (!dres)
-		return ERR_PTR(-ENOMEM);
-
-	pi = scmi_get_protocol_instance(handle, protocol_id);
-	if (IS_ERR(pi)) {
-		devres_free(dres);
+	pi = __scmi_devres_get_protocol_instance(sdev, protocol_id);
+	if (IS_ERR(pi))
 		return pi;
-	}
-
-	dres->handle = handle;
-	dres->protocol_id = protocol_id;
-	devres_add(&sdev->dev, dres);
 
 	*ph = &pi->ph;
 
 	return pi->proto->ops;
 }
 
+/**
+ * scmi_devm_acquire_protocol  - Devres managed helper to get hold of a protocol
+ * @sdev: A reference to an scmi_device whose embedded struct device is to
+ *	  be used for devres accounting.
+ * @protocol_id: The protocol being requested.
+ *
+ * Get hold of a protocol accounting for its usage, possibly triggering its
+ * initialization but without getting access to its protocol specific operations
+ * and handle.
+ *
+ * Being a devres based managed method, protocol hold will be automatically
+ * released, and possibly de-initialized on last user, once the SCMI driver
+ * owning the scmi_device is unbound from it.
+ *
+ * Return: 0 on SUCCESS
+ */
+static int __must_check scmi_devm_acquire_protocol(struct scmi_device *sdev,
+						   u8 protocol_id)
+{
+	struct scmi_protocol_instance *pi;
+
+	pi = __scmi_devres_get_protocol_instance(sdev, protocol_id);
+	if (IS_ERR(pi))
+		return PTR_ERR(pi);
+
+	return 0;
+}
+
 static int scmi_devm_protocol_match(struct device *dev, void *res, void *data)
 {
 	struct scmi_protocol_devres *dres = res;
@@ -1076,6 +1115,7 @@ static int scmi_probe(struct platform_device *pdev)
 	handle = &info->handle;
 	handle->dev = info->dev;
 	handle->version = &info->version;
+	handle->devm_acquire_protocol = scmi_devm_acquire_protocol;
 	handle->devm_get_protocol = scmi_devm_get_protocol;
 	handle->devm_put_protocol = scmi_devm_put_protocol;
 
diff --git a/include/linux/scmi_protocol.h b/include/linux/scmi_protocol.h
index bbcb2d999068..b3ca2969c29a 100644
--- a/include/linux/scmi_protocol.h
+++ b/include/linux/scmi_protocol.h
@@ -595,9 +595,15 @@ struct scmi_notify_ops {
  * @sensor_ops: pointer to set of sensor protocol operations
  * @reset_ops: pointer to set of reset protocol operations
  * @voltage_ops: pointer to set of voltage protocol operations
  * @devm_get_protocol: devres managed method to acquire a protocol and get specific
  *		       operations and a dedicated protocol handler
  * @devm_put_protocol: devres managed method to release a protocol
  * @notify_ops: pointer to set of notifications related operations
  * @perf_priv: pointer to private data structure specific to performance
  *	protocol(for internal use only)
@@ -624,6 +630,8 @@ struct scmi_handle {
 	const struct scmi_reset_ops *reset_ops;
 	const struct scmi_voltage_ops *voltage_ops;
 
+	int __must_check (*devm_acquire_protocol)(struct scmi_device *sdev,
+						  u8 proto);
 	const void __must_check *
 		(*devm_get_protocol)(struct scmi_device *sdev, u8 proto,
 				     struct scmi_protocol_handle **ph);
@@ -595,9 +595,15 @@ struct scmi_notify_ops {
  * @reset_ops: pointer to set of reset protocol operations
  * @voltage_ops: pointer to set of voltage protocol operations
- * @devm_get_protocol: devres managed method to acquire a protocol and get specific
- *		       operations and a dedicated protocol handler
- * @devm_put_protocol: devres managed method to release a protocol
+ * @devm_acquire_protocol: devres managed method to get hold of a protocol,
+ *			   causing its initialization and related resource
+ *			   accounting
+ * @devm_get_protocol: devres managed method to acquire a protocol, causing
+ *		       its initialization and resource accounting, while getting
+ *		       protocol specific operations and a dedicated protocol
+ *		       handler
+ * @devm_put_protocol: devres managed method to release a protocol acquired
+ *		       with devm_acquire/get_protocol
  * @notify_ops: pointer to set of notifications related operations
  * @perf_priv: pointer to private data structure specific to performance
-- 
2.17.1


_______________________________________________
linux-arm-kernel mailing list
linux-arm-kernel@lists.infradead.org
http://lists.infradead.org/mailman/listinfo/linux-arm-kernel

  parent reply	other threads:[~2021-02-02 22:18 UTC|newest]

Thread overview: 58+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2021-02-02 22:15 [PATCH v6 0/37] SCMI vendor protocols and modularization Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 01/37] firmware: arm_scmi: review protocol registration interface Cristian Marussi
2021-03-08  4:38   ` Sudeep Holla
2021-03-08 11:24     ` Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 02/37] firmware: arm_scmi: introduce protocol handle definitions Cristian Marussi
2021-03-08  5:50   ` Sudeep Holla
2021-03-08 11:37     ` Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 03/37] firmware: arm_scmi: introduce devres get/put protocols operations Cristian Marussi
2021-02-02 22:15 ` Cristian Marussi [this message]
2021-02-02 22:15 ` [PATCH v6 05/37] firmware: arm_scmi: make notifications aware of protocols users Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 06/37] firmware: arm_scmi: introduce new devres notification ops Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 07/37] firmware: arm_scmi: refactor events registration Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 08/37] firmware: arm_scmi: convert events registration to protocol handles Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 09/37] firmware: arm_scmi: add new protocol handle core xfer ops Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 10/37] firmware: arm_scmi: add helper to access revision area memory Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 11/37] firmware: arm_scmi: port Base protocol to new interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 12/37] firmware: arm_scmi: port Perf protocol to new protocols interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 13/37] cpufreq: scmi: port driver to the new scmi_perf_proto_ops interface Cristian Marussi
2021-02-03  3:03   ` Viresh Kumar
2021-02-03 12:10     ` Cristian Marussi
2021-02-04 11:43       ` Sudeep Holla
2021-02-04  2:14   ` Viresh Kumar
2021-02-02 22:15 ` [PATCH v6 14/37] firmware: arm_scmi: remove legacy scmi_perf_ops protocol interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 15/37] firmware: arm_scmi: port Power protocol to new protocols interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 16/37] firmware: arm_scmi: port GenPD driver to the new scmi_power_proto_ops interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 17/37] firmware: arm_scmi: remove legacy scmi_power_ops protocol interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 18/37] firmware: arm_scmi: port Clock protocol to new protocols interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 19/37] clk: scmi: port driver to the new scmi_clk_proto_ops interface Cristian Marussi
2021-03-10 14:19   ` Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 20/37] firmware: arm_scmi: remove legacy scmi_clk_ops protocol interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 21/37] firmware: arm_scmi: port Reset protocol to new protocols interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 22/37] reset: reset-scmi: port driver to the new scmi_reset_proto_ops interface Cristian Marussi
2021-03-03 14:46   ` Philipp Zabel
2021-02-02 22:15 ` [PATCH v6 23/37] firmware: arm_scmi: remove legacy scmi_reset_ops protocol interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 24/37] firmware: arm_scmi: port Sensor protocol to new protocols interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 25/37] hwmon: (scmi) port driver to the new scmi_sensor_proto_ops interface Cristian Marussi
2021-02-03  1:30   ` Guenter Roeck
2021-02-02 22:15 ` [PATCH v6 26/37] firmware: arm_scmi: remove legacy scmi_sensor_ops protocol interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 27/37] firmware: arm_scmi: port SystemPower protocol to new protocols interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 28/37] firmware: arm_scmi: port Voltage " Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 29/37] regulator: scmi: port driver to the new scmi_voltage_proto_ops interface Cristian Marussi
2021-02-03 13:24   ` Mark Brown
2021-02-03 14:39     ` Cristian Marussi
2021-02-03 16:18   ` Mark Brown
2021-02-02 22:15 ` [PATCH v6 30/37] firmware: arm_scmi: remove legacy scmi_voltage_ops protocol interface Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 31/37] firmware: arm_scmi: make references to handle const Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 32/37] firmware: arm_scmi: cleanup legacy protocol init code Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 33/37] firmware: arm_scmi: cleanup unused core xfer wrappers Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 34/37] firmware: arm_scmi: cleanup events registration transient code Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 35/37] firmware: arm_scmi: make notify_priv really private Cristian Marussi
2021-03-08  6:12   ` Sudeep Holla
2021-03-08 11:39     ` Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 36/37] firmware: arm_scmi: add protocol modularization support Cristian Marussi
2021-03-08  7:34   ` Sudeep Holla
2021-03-09  8:04     ` Cristian Marussi
2021-02-02 22:15 ` [PATCH v6 37/37] firmware: arm_scmi: add dynamic scmi devices creation Cristian Marussi
2021-03-15  8:33   ` Sudeep Holla
2021-03-15  9:03     ` Cristian Marussi

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20210202221555.41167-5-cristian.marussi@arm.com \
    --to=cristian.marussi@arm.com \
    --cc=Jonathan.Cameron@Huawei.com \
    --cc=etienne.carriere@linaro.org \
    --cc=f.fainelli@gmail.com \
    --cc=james.quinlan@broadcom.com \
    --cc=linux-arm-kernel@lists.infradead.org \
    --cc=linux-kernel@vger.kernel.org \
    --cc=lukasz.luba@arm.com \
    --cc=souvik.chakravarty@arm.com \
    --cc=sudeep.holla@arm.com \
    --cc=thara.gopinath@linaro.org \
    --cc=vincent.guittot@linaro.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).